import {
  Directive,
  OnInit,
  OnChanges,
  OnDestroy,
  Input,
  ElementRef,
  HostListener,
  ComponentFactoryResolver,
  ApplicationRef,
  Injector,
  EmbeddedViewRef,
  ComponentRef,
  TemplateRef,
  Renderer2,
  SimpleChanges
} from '@angular/core';
import { DropdownComponent } from './dropdown.component';

@Directive({
  selector: '[cmsDropdown]'
})
export class DropdownDirective implements OnInit, OnChanges, OnDestroy {
  @Input() content: TemplateRef<any>;
  @Input() className: string;
  @Input() hasResults: boolean;
  @Input() list = true;
  @Input() autoClose = true;
  @Input() isTypeahead = false;

  active = false;
  classes: string;
  currentTargetId: string;
  componentRef: ComponentRef<DropdownComponent>;

  constructor(
    private el: ElementRef,
    private renderer: Renderer2,
    private componentFactoryResolver: ComponentFactoryResolver,
    private appRef: ApplicationRef,
    private injector: Injector) {
  }

  @HostListener('click') onToggleClick(): void {
    if (this.isTypeahead) {
      return;
    }

    if (this.active) {
      this.removeDropdown();
    } else {
      this.toggleDropdown();
    }
  }

  @HostListener('focus', ['$event']) onFocusEvent(event: any): void {
    if (!this.isTypeahead || this.active) {
      return;
    }

    this.currentTargetId = event.target.id;

    if (this.hasResults) {
      this.toggleDropdown();
    }
  }

  @HostListener('blur') onBlurEvent(): void {
    if (!this.isTypeahead) {
      return;
    }

    this.currentTargetId = null;
  }

  @HostListener('document:click', ['$event']) onDocumentClick(event: any): void {
    if (this.active && this.autoClose && !this.el.nativeElement.contains(event.target)) {
      this.removeDropdown();
    }
  }

  ngOnInit(): void {
    this.setupAccessibility();
  }

  ngOnChanges(changes: SimpleChanges): void {
    if (this.currentTargetId === this.el.nativeElement.id &&
      this.isTypeahead && changes && changes.hasResults && changes.hasResults.currentValue === true) {
      this.toggleDropdown();
    } else if (this.isTypeahead && changes && changes.hasResults && changes.hasResults.currentValue === false) {
      this.removeDropdown();
    }
  }

  ngOnDestroy(): void {
    this.removeDropdown();
  }

  private removeDropdown(): void {
    this.active = false;
    this.renderer.setAttribute(this.el.nativeElement, 'aria-expanded', 'false');

    if (this.componentRef) {
      this.appRef.detachView(this.componentRef.hostView);
      this.componentRef.destroy();
    }
  }

  private toggleDropdown(): void {
    if (this.isTypeahead && this.currentTargetId !== this.el.nativeElement.id ) {
      return;
    }

    this.active = true;
    this.renderer.setAttribute(this.el.nativeElement, 'aria-expanded', 'true');

    this.componentRef = this.componentFactoryResolver
      .resolveComponentFactory(DropdownComponent)
      .create(this.injector);

    this.componentRef.instance.content = this.content;
    this.componentRef.instance.className = this.className;
    this.componentRef.instance.list = this.list;
    this.componentRef.instance.labelledBy = this.el.nativeElement.id;

    // OnPush does not see these changes, so we need to force
    this.componentRef.changeDetectorRef.markForCheck();
    this.componentRef.changeDetectorRef.detectChanges();

    this.appRef.attachView(this.componentRef.hostView);

    const ele = (this.componentRef.hostView as EmbeddedViewRef<any>).rootNodes[0] as HTMLElement;

    this.el.nativeElement.parentNode.appendChild(ele);
  }

  private setupAccessibility(): void {
    this.renderer.setAttribute(this.el.nativeElement, 'aria-haspopup', 'true');
    this.renderer.setAttribute(this.el.nativeElement, 'aria-expanded', 'false');
  }
}
