import {
  Directive,
  OnChanges,
  Input,
  ElementRef,
  ComponentRef,
  TemplateRef,
  Renderer2,
  ComponentFactoryResolver,
  ApplicationRef,
  Injector,
  EmbeddedViewRef,
  SimpleChanges,
  OnInit
} from '@angular/core';
import { ModalComponent } from './modal.component';
import { ModalBackdropComponent } from '../modal-backdrop/modal-backdrop.component';
import { WindowService } from '../services/window.service';

export interface ModalOptions {
  open: boolean;
}

@Directive({
  selector: '[cmsModal]'
})
export class ModalDirective implements OnChanges, OnInit {
  @Input() content: TemplateRef<any>;
  @Input() modalOptions: ModalOptions;

  componentRef: ComponentRef<ModalComponent>;
  backdropComponentRef: ComponentRef<ModalBackdropComponent>;
  body: HTMLElement;

  constructor(
    private el: ElementRef,
    private renderer: Renderer2,
    private windowService: WindowService,
    private componentFactoryResolver: ComponentFactoryResolver,
    private appRef: ApplicationRef,
    private injector: Injector) {}

  ngOnInit(): void {
    this.body = this.windowService.window.document.body;
  }

  ngOnChanges(changes: SimpleChanges): void {
    if (!changes || !changes.modalOptions || changes.modalOptions.firstChange) {
      return;
    }
    if (!this.componentRef && changes.modalOptions.currentValue && changes.modalOptions.currentValue.open) {
      this.createModal();
    }

    if (changes.modalOptions.currentValue && !changes.modalOptions.currentValue.open) {
      this.removeModal();
    }
  }

  private removeModal(): void {
    if (this.backdropComponentRef) {
      this.appRef.detachView(this.backdropComponentRef.hostView);
      this.backdropComponentRef.destroy();
      delete this.backdropComponentRef;
    }
    if (this.componentRef) {
      this.appRef.detachView(this.componentRef.hostView);
      this.componentRef.destroy();
      delete this.componentRef;
      this.renderer.removeClass(this.body, 'modal-open');
    }
  }

  private createModal(): void {
    this.renderer.addClass(this.el.nativeElement, 'in');
    this.renderer.addClass(this.body, 'modal-open');

    this.componentRef = this.componentFactoryResolver
      .resolveComponentFactory(ModalComponent)
      .create(this.injector);

    this.backdropComponentRef = this.componentFactoryResolver
      .resolveComponentFactory(ModalBackdropComponent)
      .create(this.injector);

    this.componentRef.instance.content = this.content;

    // OnPush does not see these changes, so we need to force
    this.componentRef.changeDetectorRef.markForCheck();
    this.componentRef.changeDetectorRef.detectChanges();

    this.appRef.attachView(this.componentRef.hostView);
    const ele = (this.componentRef.hostView as EmbeddedViewRef<any>).rootNodes[0] as HTMLElement;
    this.renderer.appendChild(this.body, ele);

    this.appRef.attachView(this.backdropComponentRef.hostView);
    const backdropElement = (this.backdropComponentRef.hostView as EmbeddedViewRef<any>).rootNodes[0] as HTMLElement;
    this.renderer.appendChild(this.body, backdropElement);
  }
}
