import {
  Directive,
  OnDestroy,
  Input,
  ElementRef,
  HostListener,
  Renderer2,
  ComponentFactoryResolver,
  ApplicationRef,
  Injector,
  EmbeddedViewRef,
  ComponentRef
} from '@angular/core';
import { TooltipComponent } from './tooltip.component';
import { WindowService } from '../services/window.service';

@Directive({
  selector: '[cmsTooltip]'
})
export class TooltipDirective implements OnDestroy {
  @Input() text: string;
  @Input() body: boolean;
  @Input() position: string;
  @Input() tooltipHeight = 40;
  @Input() disable = false;
  @Input() offset: string;

  tooltip: any;
  componentRef: ComponentRef<TooltipComponent>;

  constructor(
    private el: ElementRef,
    private renderer: Renderer2,
    private windowService: WindowService,
    private componentFactoryResolver: ComponentFactoryResolver,
    private appRef: ApplicationRef,
    private injector: Injector) {
  }

  @HostListener('mouseenter') onMouseEnter(): void {
    if (!this.disable && this.text) {
      this.createTooltip();
    }
  }

  @HostListener('mouseleave') onMouseLeave(): void {
    this.removeTooltip();
  }

  ngOnDestroy(): void {
    this.removeTooltip();
  }

  private removeTooltip(): void {
    if (this.componentRef) {
      this.appRef.detachView(this.componentRef.hostView);
      this.componentRef.destroy();
    }
  }

  private createTooltip(): void {
    this.componentRef = this.componentFactoryResolver
      .resolveComponentFactory(TooltipComponent)
      .create(this.injector);

    // set component inputs
    this.componentRef.instance.body = this.body;
    this.componentRef.instance.position = this.position;
    this.componentRef.instance.text = this.text;
    this.componentRef.instance.offset = this.offset;

    // OnPush does not see these changes, so we need to force
    this.componentRef.changeDetectorRef.markForCheck();
    this.componentRef.changeDetectorRef.detectChanges();

    this.appRef.attachView(this.componentRef.hostView);

    const ele = (this.componentRef.hostView as EmbeddedViewRef<any>).rootNodes[0] as HTMLElement;

    if (this.body) {
      this.windowService.window.document.body.appendChild(ele);
      this.tooltip = this.windowService.window.document.body.querySelector('.tool-tip');
      this.setTooltipPosition();
    } else {
      this.el.nativeElement.appendChild(ele);
      this.tooltip = this.el.nativeElement.querySelector('.tool-tip');
    }
  }

  private setTooltipPosition(): void {
    let eleRect: any;
    if (this.el.nativeElement.tagName === 'BUTTON') {
      eleRect = this.el.nativeElement.children[0].getBoundingClientRect();
    } else {
      eleRect = this.el.nativeElement.getBoundingClientRect();
    }

    const tooltipRect = this.tooltip.getBoundingClientRect();

    switch (this.position) {
      case 'top':
        this.setPositionTop(eleRect, tooltipRect);
        break;
      case 'bottom':
        this.setPositionBottom(eleRect, tooltipRect);
        break;

      default:
        this.setPositionTop(eleRect, tooltipRect);
    }
  }

  private setPositionTop(eleRect: any, tooltipRect: any): void {
    if (tooltipRect.height > this.tooltipHeight) {
      const offset = eleRect.top - (tooltipRect.height - this.tooltipHeight);
      this.renderer.setStyle(this.tooltip, 'top', offset - (eleRect.height * 2) + 'px');
    } else {
      this.renderer.setStyle(this.tooltip, 'top', eleRect.top - (eleRect.height * 2) + 'px');
    }
    this.renderer.setStyle(this.tooltip, 'left', ((eleRect.width  / 2) + eleRect.left) + 'px');
  }

  private setPositionBottom(eleRect: any, tooltipRect: any): void {
    if (tooltipRect.height > this.tooltipHeight) {
      const offset = eleRect.bottom + (tooltipRect.height - this.tooltipHeight);
      this.renderer.setStyle(this.tooltip, 'top', (offset - (eleRect.height * 2)) + (eleRect.height + 10) + 'px');
    } else {
      this.renderer.setStyle(this.tooltip, 'top', (eleRect.bottom - (eleRect.height * 2)) + (eleRect.height + 10) + 'px');
    }
    this.renderer.setStyle(this.tooltip, 'left', ((eleRect.width  / 2) + eleRect.left) + 'px');
  }
}
